package java.lang;

import java.io.OutputStream;
import java.io.InputStream;
import sun.reflect.CallerSensitive;
import sun.reflect.Reflection;
import java.util.StringTokenizer;
import java.io.File;
import java.io.IOException;

public class Runtime
{
    private static Runtime currentRuntime;
    
    public static Runtime getRuntime() {
        return Runtime.currentRuntime;
    }
    
    private Runtime() {
    }
    
    public void exit(int n) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkExit(n);
        }
        Shutdown.exit(n);
    }
    
    public void addShutdownHook(Thread thread) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("shutdownHooks"));
        }
        ApplicationShutdownHooks.add(thread);
    }
    
    public boolean removeShutdownHook(Thread thread) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new RuntimePermission("shutdownHooks"));
        }
        return ApplicationShutdownHooks.remove(thread);
    }
    
    public void halt(int n) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkExit(n);
        }
        Shutdown.beforeHalt();
        Shutdown.halt(n);
    }
    
    @Deprecated
    public static void runFinalizersOnExit(boolean runFinalizersOnExit) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            try {
                securityManager.checkExit(0);
            }
            catch (SecurityException ex) {
                throw new SecurityException("runFinalizersOnExit");
            }
        }
        Shutdown.setRunFinalizersOnExit(runFinalizersOnExit);
    }
    
    public Process exec(String s) throws IOException {
        return this.exec(s, null, null);
    }
    
    public Process exec(String s, String[] array) throws IOException {
        return this.exec(s, array, null);
    }
    
    public Process exec(String str, String[] array, File file) throws IOException {
        if (str.length() == 0) {
            throw new IllegalArgumentException("Empty command");
        }
        StringTokenizer stringTokenizer = new StringTokenizer(str);
        String[] array2 = new String[stringTokenizer.countTokens()];
        int n = 0;
        while (stringTokenizer.hasMoreTokens()) {
            array2[n] = stringTokenizer.nextToken();
            ++n;
        }
        return this.exec(array2, array, file);
    }
    
    public Process exec(String[] array) throws IOException {
        return this.exec(array, null, null);
    }
    
    public Process exec(String[] array, String[] array2) throws IOException {
        return this.exec(array, array2, null);
    }
    
    public Process exec(String[] array, String[] array2, File file) throws IOException {
        return new ProcessBuilder(array).environment(array2).directory(file).start();
    }
    
    public native int availableProcessors();
    
    public native long freeMemory();
    
    public native long totalMemory();
    
    public native long maxMemory();
    
    public native void gc();
    
    private static native void runFinalization0();
    
    public void runFinalization() {
        runFinalization0();
    }
    
    public native void traceInstructions(boolean p0);
    
    public native void traceMethodCalls(boolean p0);
    
    @CallerSensitive
    public void load(String s) {
        this.load0(Reflection.getCallerClass(), s);
    }
    
    synchronized void load0(Class<?> clazz, String pathname) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkLink(pathname);
        }
        if (!new File(pathname).isAbsolute()) {
            throw new UnsatisfiedLinkError("Expecting an absolute path of the library: " + pathname);
        }
        ClassLoader.loadLibrary(clazz, pathname, true);
        return;
    }
    
    @CallerSensitive
    public void loadLibrary(String s) {
        this.loadLibrary0(Reflection.getCallerClass(), s);
    }
    
    synchronized void loadLibrary0(Class<?> clazz, String s) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkLink(s);
        }
        if (s.indexOf((int)File.separatorChar) != -1) {
            throw new UnsatisfiedLinkError("Directory separator should not appear in library name: " + s);
        }
        ClassLoader.loadLibrary(clazz, s, false);
        return;
    }
    
    @Deprecated
    public InputStream getLocalizedInputStream(InputStream inputStream) {
        return inputStream;
    }
    
    @Deprecated
    public OutputStream getLocalizedOutputStream(OutputStream outputStream) {
        return outputStream;
    }
    
    static {
        Runtime.currentRuntime = new Runtime();
    }
}
